#!/usr/bin/perl
# ------------------------------------------
# configure.pl  -  Configurador do LVS 
#                                           
#  uso: uptimesrv.pl                      
#                                          
# Escreve os arquivos /etc/mon/mon.cf,
# /root/lvs/bin/fill_table e /root/lvs/var/polling
# usando as informaes lidas de 
# /root/lvs/etc/lvs.conf
# ------------------------------------------

use Switch;

$LVSCONF = "/root/lvs/etc/lvs.conf";
$MONCF = "/etc/mon/mon.cf";
$FILLTABLE = "/root/lvs/bin/fill_table";
$ADDSERVICE = "/root/lvs/bin/add_service";
$POLLING = "/root/lvs/var/polling";

$LVSCONF = $ARGV[0] if (@ARGV);

&read();
&verifyInfo();
&criHLinha();
&writeMonCF();
&writeFillTable();
&writePolling();

sub writeMonCF {
   open(MON, ">$MONCF");
   print MON "cfbasedir = $cfbasedir\n";
   print MON "alertdir = $alertdir\n";
   print MON "mondir = $mondir\n";
   print MON "maxprocs = 20\nhistlength = 100\nrandstart = 30s\n\n";
   my @ips = keys %HLinha;
   for (my $i = 1; $i <= $#ips + 1; $i++) {
      print MON "hostgroup server$i $ips[$i - 1]\n\n";
   }
   for (my $i = 1; $i <= $#ips + 1; $i++) {
      print MON "watch server$i\n";
      my %servList = @{$HLinha{$ips[$i - 1]}};
      foreach $service (keys %servList) {
	 print MON "\tservice $service\n";
	 print MON "\t\tinterval 10s\n";
	 if ($servList{$service} eq "tcp") {
	    print MON "\t\tmonitor tcp.monitor -p $service\n";
	    print MON "\t\tperiod wd {Sun-Sat}\n";
	    print MON "\t\t\talert lvs.alert -P tcp -V $lvshost:$service -R $ips[$i - 1] -W 1\n";
	    print MON "\t\t\tupalert lvs.alert -P tcp -V $lvshost:$service -R $ips[$i - 1] -W 1\n";
	 } else {
	    print MON "\t\tmonitor fping.monitor\n";
	    print MON "\t\tperiod wd {Sun-Sat}\n";
	    print MON "\t\t\talert lvs.alert -P udp -V $lvshost:$service -R $ips[$i - 1] -W 1\n";
	    print MON "\t\t\tupalert lvs.alert -P udp -V $lvshost:$service -R $ips[$i - 1] -W 1\n";
	 }
	 print MON "\n";
      }
   }
   close(MON);
}

sub criHLinha {
   foreach my $service (keys %hashao) {
      my %hashinho = %{$hashao{$service}};
      @service = split(/:/, $service);
      foreach my $ip (keys %hashinho) {
	 my @servList = @{$HLinha{$ip}};
	 push @servList, ("$service[1]", "$service[0]");
	 $HLinha{$ip} = \@servList;
      }
   }
}

sub writeFillTable {
   open(FT, ">$FILLTABLE");
   foreach my $key (keys %hashao) {
      my @s = split(/:/, $key);
      foreach my $otherKey (keys %{$hashao{$key}}) {
	 print FT "$ADDSERVICE $lvshost $otherKey $s[0] $s[1] $s[2]\n";
      }
   }
   close(FT);
}

sub writePolling {
   open(POLL, ">$POLLING");
   foreach my $key (keys %hashao) {
      my @s = split(/:/, $key);
      print POLL "$s[0]:$s[1] %{$hashao{$key}}\n";
   }
   close(POLL);
}

sub verifyInfo {
   die "Impossivel abrir monfile '$monfile'\n" unless -r $monfile;
   die "lvshost deve ser um IP valido" unless (&isIp($lvshost));
   die "Diretorio cfbasedir '$cfbasedir' inexistente\n" unless -d $cfbasedir;
   die "Diretorio alertdir '$alertdir' inexistente\n" unless -d $alertdir;
   die "Diretorio mondir '$mondir' inexistente\n" unless -d $mondir;
   die "pelo menos um servico deve ser especificado\n" unless (%hashao >= 0);
}

sub read {
   open(LVS, $LVSCONF);
   @lines = <LVS>;
   close(LVS);

   for ($i = 0; $lines[$i]; $i++) {
      chomp $lines[$i];
      &splitLine();
      next unless ($line[0]);
      switch($line[0]) {
	 case /monfile/ { 
	    $monfile = $line[1];
	 }
	 case /lvshost/ { 
	    $lvshost = $line[1];
	 }
	 case /cfbasedir/ { 
	    $cfbasedir = $line[1];
	 }
	 case /alertdir/ { 
	    $alertdir = $line[1];
	 }
	 case /mondir/ { 
	    $mondir = $line[1];
	 }
	 case /service/ {
	    my $proto = $line[1];
	    my $service = $line[2];
	    my $sched = $line[3];
	    my %hashinho;
	    die "1 Arquivo mal formatado na linha ".($i + 1)."\n"
	       unless (($proto eq "tcp" || $proto eq "udp") && ($service =~ /[1-9][0-9]*/) &&
		       ($sched =~ /w?(rr)|(lc)/) && $line[4] eq '{');
	    $i += 1;
	    &splitLine();
	    while ($line[0] !~ /}/) {
	       unless ($line[0]) {
		  $i += 1;
		  &splitLine();
		  next;
	       }
	       die "2 Arquivo mal formatado na linha ".($i + 1)."\n"
		  unless (&isIp($line[0]) && $line[1] =~ /[1-9][0-9]?/);
	       $hashinho{$line[0]} = $line[1];
	       $i += 1;
	       &splitLine();
	    }
	    $hashao{"$proto:$service:$sched"} = \%hashinho if (%hashinho);
	 }
	 else { die "3 Arquivo mal formatado na linha ".($i + 1)."\n"; }
      }
   }
}

sub printHashao {
   print "Imprimindo Hashao:\n";
   foreach my $key (keys %hashao) {
      print "key $key\n";
      my %hashinho = %{$hashao{$key}};
      foreach my $otherKey (keys %hashinho) {
	 print "$otherKey = $hashinho{$otherKey}\n";
      }
   }
   print "\n";
}

sub isIp {
   return ($_[0] =~ /([1-2]?[0-9]?[0-9]\.){3}[1-2]?[0-9]?[0-9]/);
}

sub splitLine {
   $lines[$i] =~ s/^\s+//;
   @line = split(/\s+/, (split(/#/, $lines[$i]))[0]);
}
