#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <error.h>
#include <sys/types.h>
#include <sys/stat.h>

#define MAXNAME 256
#define MAXHOSTS 10
#define MAXSERVICES 10
#define MAXTOKEN 15

#define FILLTABLE "/root/lvs/bin/fill_table"
#define ADDSERVICE "/root/lvs/bin/add_service"

FILE *rc_file;

int header_ok;
int server;

char services[MAXSERVICES * (MAXHOSTS + 1)][3][10];
char hosts[MAXHOSTS + 1][MAXNAME];
char lvshost[MAXNAME];
int nservices[MAXHOSTS + 1];

char cfbasedir[MAXNAME];
char alertdir[MAXNAME];
char mondir[MAXNAME];
char monfile[MAXNAME];

void consume_line();
void write_services();
void write_moncf();
void read_host();

int main(int argc, char **argv) {
   char rc_name[MAXNAME];
   char token[MAXTOKEN];
   int ho[5];
   int i;

   strcpy(rc_name, "/root/lvs/etc/lvs.conf");
   if (argc == 2) strcpy(rc_name, argv[1]);
   rc_file = fopen(rc_name, "r");
   if (rc_file == NULL)
      error(1, 1, "Impossivel abrir %s para leitura", rc_name);

   nservices[0] = 0;
   server = 0;

   for (i = 0; i < 5; i++) ho[i] = 0;

   while (fscanf(rc_file, "%s", token) == 1) {
      if (token[0] == '#') consume_line();
      else if (strcmp(token, "cfbasedir") == 0) { fscanf(rc_file, "%s", cfbasedir); ho[0] = 1; }
      else if (strcmp(token, "alertdir") == 0) { fscanf(rc_file, "%s", alertdir); ho[1] = 1; }
      else if (strcmp(token, "mondir") == 0) { fscanf(rc_file, "%s", mondir); ho[2] = 1; }
      else if (strcmp(token, "monfile") == 0) { fscanf(rc_file, "%s", monfile); ho[3] = 1; }
      else if (strcmp(token, "lvshost") == 0) { fscanf(rc_file, "%s", lvshost); ho[4] = 1; }
      else if (strcmp(token, "host") == 0) read_host();
      else error(1, 1, "Arquivo em formato inesperado");
   }

   for (i = 0; i < 5; i++)
      if (ho[i] == 0) error(1, 1, "Arquivo em formato inesperado");

   write_moncf();
   write_services();

   return 0;
}

void consume_line() {
   int c;
   while ((c = getc(rc_file)) != '\n' && c != EOF);
}

void read_host() {
   int i, index;
   char tok[MAXTOKEN];
   char hostname[MAXTOKEN];

   server++;
   index = nservices[server - 1];
   fscanf(rc_file, "%s", hosts[server]);
   fscanf(rc_file, "%s", tok);
   while (tok[0] != '{') {
      if (tok[0] == '#') consume_line();
      else error(1, 1, "Arquivo em formato inesperado");
      if (fscanf(rc_file, "%s", tok) == 0)
         error(1, 1, "Arquivo em formato inesperado");
   }
   while (1) {
      fscanf(rc_file, "%s", tok);
      if (tok[0] != '#') break;
      consume_line();
   }

   for (i = 0; tok[0] != '}'; i = (i + 1) % 3) {
      if (i == 0 && (strcmp(tok, "tcp") != 0 && strcmp(tok, "udp")) != 0)
	    error(1, 1, "Arquivo em formato inesperado");

      strcpy(services[index][i], tok);
      fscanf(rc_file, "%s", tok);
      while (tok[0] == '#') {
	 consume_line();
	 fscanf(rc_file, "%s", tok);
      }

      if (i == 2) index++;
   }

   nservices[server] = index;
}

void write_moncf() {
   FILE *moncf_file = fopen(monfile, "w");
   char hostname[MAXTOKEN];
   int i, j;

   if (moncf_file == NULL)
      error(1, 1, "Impossivel abrir %s para escrita", monfile);
   fprintf(moncf_file, "cfbasedir = %s\n", cfbasedir);
   fprintf(moncf_file, "alertdir = %s\n", alertdir);
   fprintf(moncf_file, "mondir = %s\n", mondir);
   fprintf(moncf_file, "maxprocs = 20\n");
   fprintf(moncf_file, "histlength = 100\n");
   fprintf(moncf_file, "randstart = 30s\n");

   for (i = 1; i <= server; i++)
      fprintf(moncf_file, "hostgroup server%d %s\n", i, hosts[i]);

   for (i = 1; i <= server; i++) {
      fprintf(moncf_file, "\nwatch server%d\n", i);
      for (j = nservices[i - 1]; j < nservices[i]; j++) {
	 fprintf(moncf_file, "\tservice %s\n", services[j][1]);
	 fprintf(moncf_file, "\t\tinterval 10s\n");
	 if (strcmp(services[j][0], "tcp") == 0)
	    fprintf(moncf_file, "\t\tmonitor tcp.monitor -p %s",
	    		     services[j][1]);
	 else fprintf(moncf_file, "\t\tmonitor fping.monitor");
	 fprintf(moncf_file, "\n\t\tperiod wd {Sun-Sat}\n"
			     "\t\t\talert lvs.alert -P %s -V %s:%s -R %s -W 1\n"
			     "\t\t\tupalert lvs.alert -P %s -V %s:%s -R %s -W 1\n",
			     services[j][0], lvshost, services[j][1], hosts[i],
			     services[j][0], lvshost, services[j][1], hosts[i]);
      }
   }

   fclose(moncf_file);
}

void write_services() {
   FILE *services_file;
   int i, j;

   services_file = fopen(FILLTABLE, "w");
   if (services_file == NULL)
      error(1, 1, "Impossivel abrir %s para escrita\n", FILLTABLE);

   for (i = 1; i <= server; i++)
      for (j = nservices[i - 1]; j < nservices[i]; j++)
	 fprintf(services_file, "%s %s %s %s %s %s\n", ADDSERVICE, lvshost, hosts[i],
	       services[j][0], services[j][1], services[j][2]);

   fclose(services_file);
   chmod(FILLTABLE, 00700);
}
